from questionary.prompts import autocomplete
from questionary.prompts import confirm
from questionary.prompts import text
from questionary.prompts import select
from questionary.prompts import rawselect
from questionary.prompts import password
from questionary.prompts import checkbox
from questionary.prompts import path

AVAILABLE_PROMPTS = {
    "autocomplete": autocomplete.autocomplete,
    "confirm": confirm.confirm,
    "text": text.text,
    "select": select.select,
    "rawselect": rawselect.rawselect,
    "password": password.password,
    "checkbox": checkbox.checkbox,
    "path": path.path,
    # backwards compatible names
    "list": select.select,
    "rawlist": rawselect.rawselect,
    "input": text.text,
}


def prompt_by_name(name):
    return AVAILABLE_PROMPTS.get(name)



from prompt_toolkit.output import ColorDepth
from typing import Any, Dict, Optional, Iterable, Mapping

from questionary import utils
from questionary.constants import DEFAULT_KBI_MESSAGE
from questionary.prompts import AVAILABLE_PROMPTS, prompt_by_name


class PromptParameterException(ValueError):
    def __init__(self, message: str, errors: Optional[BaseException] = None) -> None:
        # Call the base class constructor with the parameters it needs
        super().__init__("You must provide a `%s` value" % message, errors)


def prompt(
    questions: Iterable[Mapping[str, Any]],
    answers: Optional[Mapping[str, Any]] = None,
    patch_stdout: bool = False,
    true_color: bool = False,
    kbi_msg: str = DEFAULT_KBI_MESSAGE,
    **kwargs: Any,
) -> Dict[str, Any]:
    """Prompt the user for input on all the questions.

    Catches keyboard interrupts and prints a message."""

    try:
        return unsafe_prompt(questions, answers, patch_stdout, true_color, **kwargs)
    except KeyboardInterrupt:
        print("")
        print(kbi_msg)
        print("")
        return {}


def unsafe_prompt(
    questions: Iterable[Mapping[str, Any]],
    answers: Optional[Mapping[str, Any]] = None,
    patch_stdout: bool = False,
    true_color: bool = False,
    **kwargs: Any,
) -> Dict[str, Any]:
    """Prompt the user for input on all the questions.

    Won't catch keyboard interrupts.

    Raises:
        KeyboardInterrupt: raised on keyboard interrupt
    """

    if isinstance(questions, dict):
        questions = [questions]

    answers = dict(answers or {})

    for question_config in questions:
        question_config = dict(question_config)
        # import the question
        if "type" not in question_config:
            raise PromptParameterException("type")
        if "name" not in question_config:
            raise PromptParameterException("name")

        choices = question_config.get("choices")
        if choices is not None and callable(choices):
            question_config["choices"] = choices(answers)

        _kwargs = kwargs.copy()
        _kwargs.update(question_config)

        _type = _kwargs.pop("type")
        _filter = _kwargs.pop("filter", None)
        name = _kwargs.pop("name")
        when = _kwargs.pop("when", None)

        if true_color:
            _kwargs["color_depth"] = ColorDepth.TRUE_COLOR

        if when:
            # at least a little sanity check!
            if callable(question_config["when"]):
                try:
                    if not question_config["when"](answers):
                        continue
                except Exception as e:
                    raise ValueError(
                        "Problem in 'when' check of {} " "question: {}".format(name, e)
                    )
            else:
                raise ValueError(
                    "'when' needs to be function that accepts a dict argument"
                )
        if _filter:
            # at least a little sanity check!
            if not callable(_filter):
                raise ValueError(
                    "'filter' needs to be function that accepts an argument"
                )

        if callable(question_config.get("default")):
            _kwargs["default"] = question_config["default"](answers)

        create_question_func = prompt_by_name(_type)

        if not create_question_func:
            raise ValueError(
                "No question type '{}' found. "
                "Known question types are {}."
                "".format(_type, ", ".join(AVAILABLE_PROMPTS))
            )

        missing_args = list(utils.missing_arguments(create_question_func, _kwargs))
        if missing_args:
            raise PromptParameterException(missing_args[0])

        question = create_question_func(**_kwargs)

        answer = question.unsafe_ask(patch_stdout)

        if answer is not None:
            if _filter:
                try:
                    answer = _filter(answer)
                except Exception as e:
                    raise ValueError(
                        "Problem processing 'filter' of {} "
                        "question: {}".format(name, e)
                    )
            answers[name] = answer

    return answers



from prompt_toolkit import PromptSession
from prompt_toolkit.formatted_text import to_formatted_text
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.styles import merge_styles, Style
from typing import Optional, Any

from questionary.constants import (
    NO_OR_YES,
    YES,
    NO,
    YES_OR_NO,
    DEFAULT_STYLE,
    DEFAULT_QUESTION_PREFIX,
)
from questionary.question import Question


def confirm(
    message: str,
    default: bool = True,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    **kwargs: Any,
) -> Question:
    """Prompt the user to confirm or reject.

    This question type can be used to prompt the user for a confirmation
    of a yes-or-no question. If the user just hits enter, the default
    value will be returned.

    Args:
        message: Question text

        default: Default value will be returned if the user just hits
                 enter.

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    merged_style = merge_styles([DEFAULT_STYLE, style])

    status = {"answer": None}

    def get_prompt_tokens():
        tokens = []

        tokens.append(("class:qmark", qmark))
        tokens.append(("class:question", " {} ".format(message)))

        if status["answer"] is not None:
            answer = " {}".format(YES if status["answer"] else NO)
            tokens.append(("class:answer", answer))
        else:
            instruction = " {}".format(YES_OR_NO if default else NO_OR_YES)
            tokens.append(("class:instruction", instruction))

        return to_formatted_text(tokens)

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style="class:aborting")

    @bindings.add("n")
    @bindings.add("N")
    def key_n(event):
        status["answer"] = False
        event.app.exit(result=False)

    @bindings.add("y")
    @bindings.add("Y")
    def key_y(event):
        status["answer"] = True
        event.app.exit(result=True)

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):
        status["answer"] = default
        event.app.exit(result=default)

    @bindings.add(Keys.Any)
    def other(event):
        """Disallow inserting other text."""
        pass

    return Question(
        PromptSession(
            get_prompt_tokens, key_bindings=bindings, style=merged_style, **kwargs
        ).app
    )



from typing import Any, Optional, List, Tuple

from prompt_toolkit.document import Document
from prompt_toolkit.shortcuts.prompt import PromptSession
from prompt_toolkit.styles import Style, merge_styles
from prompt_toolkit.lexers import Lexer, SimpleLexer

from questionary.constants import (
    DEFAULT_QUESTION_PREFIX,
    DEFAULT_STYLE,
    INSTRUCTION_MULTILINE,
)
from questionary.prompts.common import build_validator
from questionary.question import Question


def text(
    message: str,
    default: str = "",
    validate: Any = None,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    multiline: bool = False,
    instruction: Optional[str] = None,
    lexer: Optional[Lexer] = None,
    **kwargs: Any,
) -> Question:
    """Prompt the user to enter a free text message.

    This question type can be used to prompt the user for some text input.

    Args:
        message: Question text

        default: Default value will be returned if the user just hits
                 enter.

        validate: Require the entered value to pass a validation. The
                  value can not be submited until the validator accepts
                  it (e.g. to check minimum password length).

                  This can either be a function accepting the input and
                  returning a boolean, or an class reference to a
                  subclass of the prompt toolkit Validator class.

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

        multiline: If `True`, multiline input will be enabled.

        instruction: Write instructions for the user if needed. If `None`
                     and `multiline=True`, some instructions will appear.

        lexer: Supply a valid lexer to style the answer. Leave empty to
               use a simple one by default.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    merged_style = merge_styles([DEFAULT_STYLE, style])
    lexer = lexer or SimpleLexer("class:answer")
    validator = build_validator(validate)

    if instruction is None and multiline:
        instruction = INSTRUCTION_MULTILINE

    def get_prompt_tokens() -> List[Tuple[str, str]]:
        result = [("class:qmark", qmark), ("class:question", " {} ".format(message))]
        if instruction:
            result.append(("class:instruction", " {} ".format(instruction)))
        return result

    p = PromptSession(
        get_prompt_tokens,
        style=merged_style,
        validator=validator,
        lexer=lexer,
        multiline=multiline,
        **kwargs,
    )
    p.default_buffer.reset(Document(default))

    return Question(p.app)



# -*- coding: utf-8 -*-

from typing import Any, Dict, Sequence, Optional, Union

from prompt_toolkit.application import Application
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.styles import Style, merge_styles

from questionary import utils
from questionary.constants import DEFAULT_QUESTION_PREFIX, DEFAULT_STYLE
from questionary.prompts import common
from questionary.prompts.common import Choice, InquirerControl, Separator
from questionary.question import Question


def select(
    message: str,
    choices: Sequence[Union[str, Choice, Dict[str, Any]]],
    default: Optional[Union[str, Choice, Dict[str, Any]]] = None,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    use_shortcuts: bool = False,
    use_arrow_keys: bool = True,
    use_indicator: bool = False,
    use_pointer: bool = True,
    instruction: Optional[str] = None,
    **kwargs: Any,
) -> Question:
    """Prompt the user to select one item from the list of choices.

    The user can only select one option.

    Args:
        message: Question text

        choices: Items shown in the selection, this can contain `Choice` or
                 or `Separator` objects or simple items as strings. Passing
                 `Choice` objects, allows you to configure the item more
                 (e.g. preselecting it or disabeling it).

        default: A value corresponding to a selectable item in the choices,
                 to initially set the pointer position to.


        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        instruction: A hint on how to navigate the menu.
                     It's `(Use shortcuts)` if only `use_shortcuts` is set
                     to True, `(Use arrow keys or shortcuts)` if `use_arrow_keys`
                     & `use_shortcuts` are set and `(Use arrow keys)` by default.

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

        use_indicator: Flag to enable the small indicator in front of the
                       list highlighting the current location of the selection
                       cursor.

        use_shortcuts: Allow the user to select items from the list using
                       shortcuts. The shortcuts will be displayed in front of
                       the list items.

        use_arrow_keys: Allow usage of arrow keys to select item.

        use_pointer: Flag to enable the pointer in front of the currently
                     highlighted element.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """
    if choices is None or len(choices) == 0:
        raise ValueError("A list of choices needs to be provided.")

    if use_shortcuts and len(choices) > len(InquirerControl.SHORTCUT_KEYS):
        raise ValueError(
            "A list with shortcuts supports a maximum of {} "
            "choices as this is the maximum number "
            "of keyboard shortcuts that are available. You"
            "provided {} choices!"
            "".format(len(InquirerControl.SHORTCUT_KEYS), len(choices))
        )

    merged_style = merge_styles([DEFAULT_STYLE, style])

    ic = InquirerControl(
        choices,
        default,
        use_indicator=use_indicator,
        use_shortcuts=use_shortcuts,
        use_arrow_keys=use_arrow_keys,
        use_pointer=use_pointer,
        initial_choice=default,
    )

    def get_prompt_tokens():
        # noinspection PyListCreation
        tokens = [("class:qmark", qmark), ("class:question", " {} ".format(message))]

        if ic.is_answered:
            if isinstance(ic.get_pointed_at().title, list):
                tokens.append(
                    (
                        "class:answer",
                        "".join([token[1] for token in ic.get_pointed_at().title]),
                    )
                )
            else:
                tokens.append(("class:answer", " " + ic.get_pointed_at().title))
        else:
            if instruction:
                tokens.append(("class:instruction", instruction))
            else:
                if use_shortcuts and use_arrow_keys:
                    instruction_msg = " (Use shortcuts or arrow keys)"
                elif use_shortcuts and not use_arrow_keys:
                    instruction_msg = " (Use shortcuts)"
                else:
                    instruction_msg = " (Use arrow keys)"
                tokens.append(("class:instruction", instruction_msg))

        return tokens

    layout = common.create_inquirer_layout(ic, get_prompt_tokens, **kwargs)

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style="class:aborting")

    if use_shortcuts:
        # add key bindings for choices
        for i, c in enumerate(ic.choices):
            if isinstance(c, Separator):
                continue

            # noinspection PyShadowingNames
            def _reg_binding(i, keys):
                # trick out late evaluation with a "function factory":
                # https://stackoverflow.com/a/3431699
                @bindings.add(keys, eager=True)
                def select_choice(event):
                    ic.pointed_at = i

            _reg_binding(i, c.shortcut_key)

    if use_arrow_keys or use_shortcuts is False:

        @bindings.add(Keys.Down, eager=True)
        @bindings.add("j", eager=True)
        def move_cursor_down(event):
            ic.select_next()
            while not ic.is_selection_valid():
                ic.select_next()

        @bindings.add(Keys.Up, eager=True)
        @bindings.add("k", eager=True)
        def move_cursor_up(event):
            ic.select_previous()
            while not ic.is_selection_valid():
                ic.select_previous()

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):
        ic.is_answered = True
        event.app.exit(result=ic.get_pointed_at().value)

    @bindings.add(Keys.Any)
    def other(event):
        """Disallow inserting other text. """
        pass

    return Question(
        Application(
            layout=layout,
            key_bindings=bindings,
            style=merged_style,
            **utils.used_kwargs(kwargs, Application.__init__),
        )
    )



import inspect
from prompt_toolkit import PromptSession
from prompt_toolkit.filters import IsDone, Always, Condition
from prompt_toolkit.layout import (
    FormattedTextControl,
    Layout,
    HSplit,
    ConditionalContainer,
    Window,
)
from prompt_toolkit.styles import Style, merge_styles
from prompt_toolkit.validation import Validator, ValidationError
from typing import Optional, Any, List, Dict, Union, Callable, Sequence, Tuple

from questionary.constants import (
    DEFAULT_STYLE,
    SELECTED_POINTER,
    INDICATOR_SELECTED,
    INDICATOR_UNSELECTED,
)

# This is a cut-down version of `prompt_toolkit.formatted_text.AnyFormattedText`
# which does not exist in v2 of prompt_toolkit
FormattedText = Union[
    str,
    List[Tuple[str, str]],
    List[Tuple[str, str, Callable[[Any], None]]],
    None,
]


class Choice(object):
    """One choice in a select, rawselect or checkbox."""

    title: FormattedText
    value: Optional[Any]
    disabled: Optional[str]
    checked: Optional[bool]
    shortcut_key: Optional[str]

    def __init__(
        self,
        title: FormattedText,
        value: Optional[Any] = None,
        disabled: Optional[str] = None,
        checked: Optional[bool] = False,
        shortcut_key: Optional[str] = None,
    ) -> None:
        """Create a new choice.

        Args:
            title: Text shown in the selection list.

            value: Value returned, when the choice is selected.

            disabled: If set, the choice can not be selected by the user. The
                      provided text is used to explain, why the selection is
                      disabled.

            checked: Preselect this choice when displaying the options.

            shortcut_key: Key shortcut used to select this item.
        """

        self.disabled = disabled
        self.title = title
        self.checked = checked if checked is not None else False

        if value is not None:
            self.value = value
        elif isinstance(title, list):
            self.value = "".join([token[1] for token in title])
        else:
            self.value = title

        if shortcut_key is not None:
            self.shortcut_key = str(shortcut_key)
        else:
            self.shortcut_key = None

    @staticmethod
    def build(c: Union[str, "Choice", Dict[str, Any]]) -> "Choice":
        """Create a choice object from different representations."""

        if isinstance(c, Choice):
            return c
        elif isinstance(c, str):
            return Choice(c, c)
        else:
            return Choice(
                c.get("name"),
                c.get("value"),
                c.get("disabled", None),
                c.get("checked"),
                c.get("key"),
            )


class Separator(Choice):
    """Used to space/separate choices group."""

    default_separator: str = "-" * 15
    line: str

    def __init__(self, line: Optional[str] = None) -> None:
        """Create a separator in a list.

        Args:
            line: Text to be displayed in the list, by default uses `---`.
        """

        self.line = line or self.default_separator
        super().__init__(self.line, None, "-")


class InquirerControl(FormattedTextControl):
    SHORTCUT_KEYS = [
        "1",
        "2",
        "3",
        "4",
        "5",
        "6",
        "7",
        "8",
        "9",
        "0",
        "a",
        "b",
        "c",
        "d",
        "e",
        "f",
        "g",
        "h",
        "i",
        "j",
        "k",
        "l",
        "m",
        "n",
        "o",
        "p",
        "q",
        "r",
        "s",
        "t",
        "u",
        "v",
        "w",
        "x",
        "y",
        "z",
    ]

    choices: List[Choice]
    default: Optional[Union[str, Choice, Dict[str, Any]]]
    selected_options: List[Any]
    use_indicator: bool
    use_shortcuts: bool
    use_arrow_keys: bool
    use_pointer: bool
    pointed_at: int
    is_answered: bool

    def __init__(
        self,
        choices: Sequence[Union[str, Choice, Dict[str, Any]]],
        default: Optional[Union[str, Choice, Dict[str, Any]]] = None,
        use_indicator: bool = True,
        use_shortcuts: bool = False,
        use_arrow_keys: bool = True,
        use_pointer: bool = True,
        initial_choice: Optional[Union[str, Choice, Dict[str, Any]]] = None,
        **kwargs: Any,
    ):

        self.use_indicator = use_indicator
        self.use_shortcuts = use_shortcuts
        self.use_arrow_keys = use_arrow_keys
        self.use_pointer = use_pointer
        self.default = default

        if default is not None and default not in choices:
            raise ValueError(
                f"Invalid `default` value passed. The value (`{default}`) "
                f"does not exist in the set of choices. Please make sure the "
                f"default value is one of the available choices."
            )

        if initial_choice is None:
            pointed_at = None
        elif initial_choice in choices:
            pointed_at = choices.index(initial_choice)
        else:
            raise ValueError(
                f"Invalid `initial_choice` value passed. The value "
                f"(`{initial_choice}`) does not exist in "
                f"the set of choices. Please make sure the initial value is "
                f"one of the available choices."
            )

        self.is_answered = False
        self.choices = []
        self.submission_attempted = False
        self.error_message = None
        self.selected_options = []

        self._init_choices(choices, pointed_at)
        self._assign_shortcut_keys()

        super().__init__(self._get_choice_tokens, **kwargs)

        if not self.is_selection_valid():
            raise ValueError(
                f"Invalid 'initial_choice' value ('{initial_choice}'). "
                f"It must be a selectable value."
            )

    def _is_selected(self, choice: Choice):
        return (
            choice.checked or choice.value == self.default and self.default is not None
        ) and not choice.disabled

    def _assign_shortcut_keys(self):
        available_shortcuts = self.SHORTCUT_KEYS[:]

        # first, make sure we do not double assign a shortcut
        for c in self.choices:
            if c.shortcut_key is not None:
                if c.shortcut_key in available_shortcuts:
                    available_shortcuts.remove(c.shortcut_key)
                else:
                    raise ValueError(
                        "Invalid shortcut '{}'"
                        "for choice '{}'. Shortcuts "
                        "should be single characters or numbers. "
                        "Make sure that all your shortcuts are "
                        "unique.".format(c.shortcut_key, c.title)
                    )

        shortcut_idx = 0
        for c in self.choices:
            if c.shortcut_key is None and not c.disabled:
                c.shortcut_key = available_shortcuts[shortcut_idx]
                shortcut_idx += 1

            if shortcut_idx == len(available_shortcuts):
                break  # fail gracefully if we run out of shortcuts

    def _init_choices(
        self,
        choices: Sequence[Union[str, Choice, Dict[str, Any]]],
        pointed_at: Optional[int],
    ):
        # helper to convert from question format to internal format
        self.choices = []

        if pointed_at is not None:
            self.pointed_at = pointed_at

        for i, c in enumerate(choices):
            choice = Choice.build(c)

            if self._is_selected(choice):
                self.selected_options.append(choice.value)

            if pointed_at is None and not choice.disabled:
                # find the first (available) choice
                self.pointed_at = pointed_at = i

            self.choices.append(choice)

    @property
    def choice_count(self) -> int:
        return len(self.choices)

    def _get_choice_tokens(self):
        tokens = []

        def append(index: int, choice: Choice):
            # use value to check if option has been selected
            selected = choice.value in self.selected_options

            if index == self.pointed_at:
                if self.use_pointer:
                    tokens.append(("class:pointer", " {} ".format(SELECTED_POINTER)))
                else:
                    tokens.append(("class:text", "   "))

                tokens.append(("[SetCursorPosition]", ""))
            else:
                tokens.append(("class:text", "   "))

            if isinstance(choice, Separator):
                tokens.append(("class:separator", "{}".format(choice.title)))
            elif choice.disabled:  # disabled
                if isinstance(choice.title, list):
                    tokens.append(
                        ("class:selected" if selected else "class:disabled", "- ")
                    )
                    tokens.extend(choice.title)
                else:
                    tokens.append(
                        (
                            "class:selected" if selected else "class:disabled",
                            "- {}".format(choice.title),
                        )
                    )

                tokens.append(
                    (
                        "class:selected" if selected else "class:disabled",
                        "{}".format(
                            ""
                            if isinstance(choice.disabled, bool)
                            else " ({})".format(choice.disabled)
                        ),
                    )
                )
            else:
                if self.use_shortcuts and choice.shortcut_key is not None:
                    shortcut = "{}) ".format(choice.shortcut_key)
                else:
                    shortcut = ""

                if selected:
                    if self.use_indicator:
                        indicator = INDICATOR_SELECTED + " "
                    else:
                        indicator = ""

                    tokens.append(("class:selected", "{}".format(indicator)))
                else:
                    if self.use_indicator:
                        indicator = INDICATOR_UNSELECTED + " "
                    else:
                        indicator = ""

                    tokens.append(("class:text", "{}".format(indicator)))

                if isinstance(choice.title, list):
                    tokens.extend(choice.title)
                elif selected:
                    tokens.append(
                        ("class:selected", "{}{}".format(shortcut, choice.title))
                    )
                elif index == self.pointed_at:
                    tokens.append(
                        ("class:highlighted", "{}{}".format(shortcut, choice.title))
                    )
                else:
                    tokens.append(("class:text", "{}{}".format(shortcut, choice.title)))

            tokens.append(("", "\n"))

        # prepare the select choices
        for i, c in enumerate(self.choices):
            append(i, c)

        if self.use_shortcuts:
            tokens.append(
                (
                    "class:text",
                    "  Answer: {}" "".format(self.get_pointed_at().shortcut_key),
                )
            )
        else:
            tokens.pop()  # Remove last newline.
        return tokens

    def is_selection_a_separator(self) -> bool:
        selected = self.choices[self.pointed_at]
        return isinstance(selected, Separator)

    def is_selection_disabled(self) -> Optional[str]:
        return self.choices[self.pointed_at].disabled

    def is_selection_valid(self) -> bool:
        return not self.is_selection_disabled() and not self.is_selection_a_separator()

    def select_previous(self) -> None:
        self.pointed_at = (self.pointed_at - 1) % self.choice_count

    def select_next(self) -> None:
        self.pointed_at = (self.pointed_at + 1) % self.choice_count

    def get_pointed_at(self) -> Choice:
        return self.choices[self.pointed_at]

    def get_selected_values(self) -> List[Choice]:
        # get values not labels
        return [
            c
            for c in self.choices
            if (not isinstance(c, Separator) and c.value in self.selected_options)
        ]


def build_validator(validate: Any) -> Optional[Validator]:
    if validate:
        if inspect.isclass(validate) and issubclass(validate, Validator):
            return validate()
        elif isinstance(validate, Validator):
            return validate
        elif callable(validate):

            class _InputValidator(Validator):
                def validate(self, document):
                    verdict = validate(document.text)
                    if verdict is not True:
                        if verdict is False:
                            verdict = "invalid input"
                        raise ValidationError(
                            message=verdict, cursor_position=len(document.text)
                        )

            return _InputValidator()
    return None


def _fix_unecessary_blank_lines(ps: PromptSession) -> None:
    """This is a fix for additional empty lines added by prompt toolkit.

    This assumes the layout of the default session doesn't change, if it
    does, this needs an update."""

    default_container = ps.layout.container

    default_buffer_window = (
        default_container.get_children()[0].content.get_children()[1].content
    )

    assert isinstance(default_buffer_window, Window)
    # this forces the main window to stay as small as possible, avoiding
    # empty lines in selections
    default_buffer_window.dont_extend_height = Always()
    default_buffer_window.always_hide_cursor = Always()


def create_inquirer_layout(
    ic: InquirerControl,
    get_prompt_tokens: Callable[[], List[Tuple[str, str]]],
    **kwargs: Any,
) -> Layout:
    """Create a layout combining question and inquirer selection."""

    ps = PromptSession(get_prompt_tokens, reserve_space_for_menu=0, **kwargs)
    _fix_unecessary_blank_lines(ps)

    validation_prompt = PromptSession(bottom_toolbar=lambda: ic.error_message, **kwargs)

    return Layout(
        HSplit(
            [
                ps.layout.container,
                ConditionalContainer(Window(ic), filter=~IsDone()),
                ConditionalContainer(
                    validation_prompt.layout.container,
                    filter=Condition(lambda: ic.error_message is not None),
                ),
            ]
        )
    )


def print_formatted_text(text: str, style: Optional[str] = None, **kwargs: Any) -> None:
    """Print formatted text.

    Args:
        text: text to be printed
        style: style used for printing. uses as prompt toolkit style string, details at
            https://python-prompt-toolkit.readthedocs.io/en/master/pages/advanced_topics/styling.html#style-strings

    Example:
        print_formatted_text("Hello World!", style="bold italic fg:darkred")"""
    from prompt_toolkit import print_formatted_text as pt_print
    from prompt_toolkit.formatted_text import FormattedText as FText

    if style is not None:
        text_style = Style([("text", style)])
    else:
        text_style = DEFAULT_STYLE

    pt_print(FText([("class:text", text)]), style=text_style, **kwargs)



# noinspection PyUnresolvedReferences
from prompt_toolkit.validation import Validator, ValidationError

import questionary.version
from questionary.form import Form
from questionary.form import form
from questionary.prompt import prompt, unsafe_prompt
from questionary.prompts.common import Choice
from questionary.prompts.common import Separator
from questionary.prompts.common import print_formatted_text as print
from questionary.question import Question

# import the shortcuts to create single question prompts
from questionary.prompts.autocomplete import autocomplete
from questionary.prompts.select import select
from questionary.prompts.checkbox import checkbox
from questionary.prompts.text import text
from questionary.prompts.path import path
from questionary.prompts.confirm import confirm
from questionary.prompts.password import password
from questionary.prompts.rawselect import rawselect


__version__ = questionary.version.__version__

__all__ = [
    "Validator",
    "ValidationError",
    "Form",
    "form",
    "prompt",
    "unsafe_prompt",
    "Choice",
    "Separator",
    "Question",
    "autocomplete",
    "select",
    "checkbox",
    "text",
    "confirm",
    "password",
    "print",
    "path",
    "rawselect",
    "__version__",
]



from typing import Any, Optional

from prompt_toolkit.styles import Style

from questionary.constants import DEFAULT_QUESTION_PREFIX
from questionary.prompts import text
from questionary.question import Question


def password(
    message: str,
    default: str = "",
    validate: Any = None,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    **kwargs: Any,
) -> Question:
    """Question the user to enter a secret text not displayed in the prompt.

    This question type can be used to prompt the user for information
    that should not be shown in the command line. The typed text will be
    replaced with `*`.

    Args:
        message: Question text

        default: Default value will be returned if the user just hits
                 enter.

        validate: Require the entered value to pass a validation. The
                  value can not be submited until the validator accepts
                  it (e.g. to check minimum password length).

                  This can either be a function accepting the input and
                  returning a boolean, or an class reference to a
                  subclass of the prompt toolkit Validator class.

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    return text.text(
        message, default, validate, qmark, style, is_password=True, **kwargs
    )



from prompt_toolkit.styles import Style

# Value to display as an answer when "affirming" a confirmation question
YES = "Yes"

# Value to display as an answer when "denying" a confirmation question
NO = "No"

# Instruction text for a confirmation question (yes is default)
YES_OR_NO = "(Y/n)"

# Instruction text for a confirmation question (no is default)
NO_OR_YES = "(y/N)"

# Instruction for multiline input
INSTRUCTION_MULTILINE = "(Finish with 'Alt+Enter' or 'Esc, Enter')\n>"

# Selection token used to indicate the selection cursor in a list
SELECTED_POINTER = "»"

# Item prefix to identify selected items in a checkbox list
INDICATOR_SELECTED = "●"

# Item prefix to identify unselected items in a checkbox list
INDICATOR_UNSELECTED = "○"

# Prefix displayed in front of questions
DEFAULT_QUESTION_PREFIX = "?"

# Message shown when a user aborts a question prompt using CTRL-C
DEFAULT_KBI_MESSAGE = "Cancelled by user"

# Default message style
DEFAULT_STYLE = Style(
    [
        ("qmark", "fg:#5f819d"),  # token in front of the question
        ("question", "bold"),  # question text
        ("text", ""),  # any other text
        ("answer", "fg:#FF9D00 bold"),  # submitted answer text behind the question
        ("pointer", ""),  # pointer used in select and checkbox prompts
        ("selected", ""),  # style for a selected item of a checkbox
        ("separator", ""),  # separator in lists
        ("instruction", ""),  # user instructions for select, rawselect, checkbox
    ]
)



from typing import Any, Dict, NamedTuple, Sequence

from questionary.constants import DEFAULT_KBI_MESSAGE
from questionary.question import Question


class FormField(NamedTuple):
    key: str
    question: Question


def form(**kwargs: Question) -> "Form":
    """Create a form with multiple questions.

    The parameter name of a question will be the key for the answer in
    the returned dict."""
    return Form(*(FormField(k, q) for k, q in kwargs.items()))


class Form:
    """Multi question prompts. Questions are asked one after another.

    All the answers are returned as a dict with one entry per question."""

    form_fields: Sequence[FormField]

    def __init__(self, *form_fields: FormField) -> None:
        self.form_fields = form_fields

    def unsafe_ask(self, patch_stdout: bool = False) -> Dict[str, Any]:
        return {f.key: f.question.unsafe_ask(patch_stdout) for f in self.form_fields}

    async def unsafe_ask_async(self, patch_stdout: bool = False) -> Dict[str, Any]:
        return {
            f.key: await f.question.unsafe_ask_async(patch_stdout)
            for f in self.form_fields
        }

    def ask(
        self, patch_stdout: bool = False, kbi_msg: str = DEFAULT_KBI_MESSAGE
    ) -> Dict[str, Any]:
        try:
            return self.unsafe_ask(patch_stdout)
        except KeyboardInterrupt:
            print("")
            print(kbi_msg)
            print("")
            return {}

    async def ask_async(
        self, patch_stdout: bool = False, kbi_msg: str = DEFAULT_KBI_MESSAGE
    ) -> Dict[str, Any]:
        try:
            return await self.unsafe_ask_async(patch_stdout)
        except KeyboardInterrupt:
            print("")
            print(kbi_msg)
            print("")
            return {}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['questionary', 'questionary.prompts']

package_data = \
{'': ['*']}

install_requires = \
['prompt_toolkit>=2.0,<4.0']

setup_kwargs = {
    'name': 'questionary',
    'version': '1.8.0',
    'description': 'Python library to build pretty command line user prompts ⭐️',
    'long_description': '# questionary\n\n[![version](https://img.shields.io/pypi/v/questionary.svg)](https://pypi.org/project/questionary/)\n[![license](https://img.shields.io/pypi/l/questionary.svg)](https://pypi.org/project/questionary/)\n[![Build Status](https://travis-ci.com/tmbo/questionary.svg?branch=master)](https://travis-ci.com/tmbo/questionary)\n[![Coverage Status](https://coveralls.io/repos/github/tmbo/questionary/badge.svg?branch=master)](https://coveralls.io/github/tmbo/questionary?branch=master)\n[![Supported Python Versions](https://img.shields.io/pypi/pyversions/questionary.svg)](https://pypi.python.org/pypi/questionary)\n[![FOSSA Status](https://app.fossa.io/api/projects/git%2Bgithub.com%2Ftmbo%2Fquestionary.svg?type=shield)](https://app.fossa.io/projects/git%2Bgithub.com%2Ftmbo%2Fquestionary?ref=badge_shield)\n\n✨An easy to use python library to build pretty command line user prompts ✨\n\n![example-gif](docs/images/example.gif)\n\nYou need input from a user, e.g. how an output file should be named or if he really \nwants to execute that dangerous operation? This library will help you make the \ninput prompts easy to read and answer for the user.\n\nUsed and Supported by:\n\n[<img src="https://rasa.com/static/686aee8071dd209f198d500b1164e350/4828e/rasa.png" width="100">](https://github.com/RasaHQ/rasa)\n\n## Quickstart\n\n`questionary` can be installed using `pip`:\n\n```bash\n$ pip install questionary\n✨🎂✨\n```\n\nSatisfaction guaranteed. Let\'s create a first question:\n\n```python\nimport questionary\n\nquestionary.select(\n    "What do you want to do?",\n    choices=[\n        \'Order a pizza\',\n        \'Make a reservation\',\n        \'Ask for opening hours\'\n    ]).ask()  # returns value of selection\n```\n\nThat\'s all it takes to create a user prompt! There are different types of prompts, \nyou\'ll find examples for all of them further down.\n\n<details>\n<summary>Alternative: Building from source</summary>\n\nquestionary uses Poetry for packaging and dependency management. If you want \nto build it from source, you have to install Poetry first. \n\nThis is how it can be done:\n\n```\ncurl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python3\n```\n\nThere are several other ways to install Poetry. Please, follow \n[the official guide](https://python-poetry.org/docs/#installation) to see all possible options.\n\nTo install dependencies and `questionary` itself in editable mode execute\n```\nmake install\n```\n</details>\n\n## Documentation\n\n### Different question types\n\n<details><summary>text</summary>\n\n   A free text input for the user.\n\n   ```python\n   questionary.text("What\'s your first name").ask()\n   ```\n   ![example-gif](docs/images/text.gif)\n\n</details>\n<details><summary>password</summary>\n\n   A free text input for the user where the input is not\n   shown but replaced with `***`.\n\n   ```python\n   questionary.password("What\'s your secret?").ask()\n   ```\n\n   ![example-gif](docs/images/password.gif)\n\n</details>\n<details><summary>path</summary>\n\n   A text input for a file or directory path with autocompletion enabled.\n\n   ```python\n   questionary.path("Path to the projects version file").ask()\n   ```\n\n   ![example-gif](docs/images/path.gif)\n\n</details>\n<details><summary>confirm</summary>\n\n   A yes or no question. The user can either confirm or deny.\n\n   ```python\n   questionary.confirm("Are you amazed?").ask()\n   ```\n\n   ![example-gif](docs/images/confirm.gif)\n\n</details>\n<details><summary>select</summary>\n\n   A list of items to select a choice from. The user can pick\n   one option and confirm it.\n\n   ```python\n   questionary.select(\n       "What do you want to do?",\n       choices=[\n           "Order a pizza",\n           "Make a reservation",\n           "Ask for opening hours"\n       ]).ask()\n   ```\n\n   ![example-gif](docs/images/select.gif)\n\n</details>\n<details><summary>rawselect</summary>\n\n   A list of items to select a choice from. The user can pick\n   one option using shortcuts and confirm it.\n\n   ```python\n   questionary.rawselect(\n       "What do you want to do?",\n       choices=[\n           "Order a pizza",\n           "Make a reservation",\n           "Ask for opening hours"\n       ]).ask()\n   ```\n\n   ![example-gif](docs/images/rawselect.gif)\n\n</details>\n\n<details><summary>checkbox</summary>\n\n   A list of items to select multiple choices from. The user can pick\n   none, one or multiple options and confirm the selection.\n\n   ```python\n   questionary.checkbox(\n       \'Select toppings\',\n       choices=[\n           "foo",\n           "bar",\n           "bazz"\n       ]).ask()\n   ```\n   ![example-gif](docs/images/checkbox.gif)\n\n</details>\n\n<details><summary>autocomplete</summary>\n\n   Text input with autocomplete help.\n\n   ```python\n   questionary.autocomplete(\n       \'Choose ant specie\',\n       choices=[\n            \'Camponotus pennsylvanicus\',\n            \'Linepithema humile\',\n            \'Eciton burchellii\',\n            "Atta colombica",\n            \'Polyergus lucidus\',\n            \'Polyergus rufescens\',\n       ]).ask()\n   ```\n   ![example-gif](docs/images/autocomplete.gif)\n\n</details>\n\n### Additional Features\n<details><summary>Printing formatted text</summary>\n\nSometimes you want to spice up your printed messages a bit, `questionary.print`\nis a helper to do just that:\n\n```python\n\nquestionary.print("Hello World 🦄", style="bold italic fg:darkred")\n```\n![example-gif](docs/images/print.gif) \n\nThe style argument uses the prompt [toolkit style strings](https://python-prompt-toolkit.readthedocs.io/en/master/pages/advanced_topics/styling.html#style-strings).\n</details>\n\n<details><summary>Skipping questions using conditions</summary>\n\nSometimes it is helpful to e.g. provide a command line flag to your app\nto skip any prompts, to avoid the need for an if around any question you\ncan pass that flag when you create the question:\n\n```python\nDISABLED = True\n\nresponse = questionary.confirm("Are you amazed?").skip_if(DISABLED, default=True).ask()\n```\n\nIf the condition (in this case `DISABLED`) is `True`, the question will be\nskipped and the default value gets returned, otherwise the user will be\nprompted as usual and the default value will be ignored.\n</details>\n\n<details><summary>Alternative style to create questions using a configuration dictionary</summary>\n\nInstead of creating questions using the python functions, you can also create them using a configuration dictionary.\n```python\nquestions = [\n    {\n        \'type\': \'text\',\n        \'name\': \'phone\',\n        \'message\': "What\'s your phone number",\n    },\n    {\n        \'type\': \'confirm\',\n        \'message\': \'Do you want to continue?\',\n        \'name\': \'continue\',\n        \'default\': True,\n    }\n]\n\nanswers = prompt(questions)\n```\n\nThe returned `answers` will be a dict containing the responses, e.g. `{"phone": "0123123", "continue": False, ""}`. The questions will be prompted one after another and `prompt` will return once all of them are answered.\n\nEach configuration dictionary needs to contain the following keys:\n\n* `\'type\'` - The type of the question.\n* `\'name\'` - The name of the question (will be used as key in the `answers` dictionary)\n* `\'message\'` - Message that will be shown to the user\n\nOptional Keys:\n\n* `\'qmark\'` - Question mark to use - defaults to `?`.\n* `\'default\'` - Preselected value.\n* `\'choices\'` - List of choices (applies when `\'type\': \'select\'`) or function returning a list of choices.\n* `\'when\'` - Function checking if this question should be shown or skipped (same functionality than `.skip_if()`).\n* `\'validate\'` - Function or Validator Class performing validation (will be performed in real time as users type).\n* `filter` - Receive the user input and return the filtered value to be used inside the program. \n\n</details>\n\n<details><summary>Advanced workflow examples</summary>\nQuestionary allows creating quite complex workflows when combining all of the above concepts.\n\n``` python\nfrom questionary import Separator, prompt\nquestions = [\n    {\n        \'type\': \'confirm\',\n        \'name\': \'conditional_step\',\n        \'message\': \'Would you like the next question?\',\n        \'default\': True,\n    },\n    {\n        \'type\': \'text\',\n        \'name\': \'next_question\',\n        \'message\': \'Name this library?\',\n        # Validate if the first question was answered with yes or no\n        \'when\': lambda x: x[\'conditional_step\'],\n        # Only accept questionary as answer\n        \'validate\': lambda val: val == \'questionary\'\n    },\n    {\n        \'type\': \'select\',\n        \'name\': \'second_question\',\n        \'message\': \'Select item\',\n        \'choices\': [\n            \'item1\',\n            \'item2\',\n            Separator(),\n            \'other\',\n        ],\n    },\n    {\n        \'type\': \'text\',\n        \'name\': \'second_question\',\n        \'message\': \'Insert free text\',\n        \'when\': lambda x: x[\'second_question\'] == \'other\'\n    },\n]\nprompt(questions)\n```\n\nThe above workflow will show to the user as follows:\n1. Yes/No question `Would you like the next question?`.\n2. `Name this library?` - only shown when the first question is answered with yes\n3. A question to select an item from a list.\n4. Free text inpt if `\'other\'` is selected in step 3.\n\nDepending on the route the user took, the result will look as follows:\n\n``` python\n{ \n    \'conditional_step\': False,\n    \'second_question\': \'Testinput\'   # Free form text\n}\n```\n``` python\n{ \n    \'conditional_step\': True,\n    \'next_question\': \'questionary\',\n    \'second_question\': \'Testinput\'   # Free form text\n}\n```\n\nYou can test this workflow yourself by running the [advanced_workflow.py example](https://github.com/tmbo/questionary/blob/master/examples/advanced_workflow.py).\n\n</details>\n\n<details><summary>Styling your prompts with your favorite colors</summary>\n\nYou can customize all the colors used for the prompts. Every part of the prompt has an identifier, which you can use to style it. Let\'s create our own custom style:\n```python\nfrom prompt_toolkit.styles import Style\n\ncustom_style_fancy = Style([\n    (\'qmark\', \'fg:#673ab7 bold\'),       # token in front of the question\n    (\'question\', \'bold\'),               # question text\n    (\'answer\', \'fg:#f44336 bold\'),      # submitted answer text behind the question\n    (\'pointer\', \'fg:#673ab7 bold\'),     # pointer used in select and checkbox prompts\n    (\'highlighted\', \'fg:#673ab7 bold\'), # pointed-at choice in select and checkbox prompts\n    (\'selected\', \'fg:#cc5454\'),         # style for a selected item of a checkbox\n    (\'separator\', \'fg:#cc5454\'),        # separator in lists\n    (\'instruction\', \'\'),                # user instructions for select, rawselect, checkbox\n    (\'text\', \'\'),                       # plain text\n    (\'disabled\', \'fg:#858585 italic\')   # disabled choices for select and checkbox prompts\n])\n```\n\nTo use our custom style, we need to pass it to the question type:\n```python\nquestionary.text("What\'s your phone number", style=custom_style_fancy).ask()\n```\n\nIt is also possible to use a list of token tuples as a `Choice` title. This\nexample assumes there is a style token named `bold` in the custom style you are\nusing:\n```python\nChoice(\n    title=[\n        (\'class:text\', \'plain text \'),\n        (\'class:bold\', \'bold text\')\n    ]\n)\n```\nAs you can see it is possible to use custom style tokens for this purpose as\nwell. Note that Choices with token tuple titles will not be styled by the\n`selected` or `highlighted` tokens. If not provided, the `value` of the Choice\nwill be the text concatenated (`\'plain text bold text\'` in the above example).\n</details>\n\n## How to Contribute\n\nContributions are highly welcomed and appreciated. Every little help counts, \nso do not hesitate!\n\n1.  Check for open issues or open a fresh issue to start a discussion\n    around a feature idea or a bug. There is a [Contributor\n    Friendly](https://github.com/tmbo/questionary/issues?direction=desc&labels=good+first+issue&page=1&sort=updated&state=open)\n    tag for issues that should be ideal for people who are not very\n    familiar with the codebase yet.\n2.  Fork [the repository](https://github.com/tmbo/questionary) on\n    GitHub to start making your changes to the **master** branch (or\n    branch off of it).\n3.  Write a test which shows that the bug was fixed or that the feature\n    works as expected.\n4.  Ensure your code passes running `black questionary`.\n5.  Send a pull request and bug the maintainer until it gets merged and\n    published. 🙂\n\n## Contributors\n\n`questionary` is written and maintained by Tom Bocklisch.\n\nIt is based on the great work of [Oyetoke Toby](https://github.com/CITGuru/PyInquirer) \nas well as the work from [Mark Fink](https://github.com/finklabs/whaaaaat).\n\n## Changelog\n\n<details><summary>1.8.0 (08.11.2020)</summary>\n\n* Added additional question type `questionary.path`\n* Added possibility to validate select and checkboxes selections before submitting them\n* Added a helper to print formatted text `questionary.print`\n* Added API method to call prompt in an unsafe way\n* Hide cursor on select only showing the item marker\n\n</details>\n\n<details><summary>1.7.0 (15.10.2020)</summary>\n\n* Added support for python 3.9\n* Better UX for multiline text input\n* Allow passing custom lexer\n\n</details>\n\n<details><summary>1.6.0 (04.10.2020)</summary>\n\n* Updated black code style formatting and fixed version.\n* Fixed colour of answer for some prompts.\n* Added `py.typed` marker file.\n* Documented multiline input for devs and users and added tests.\n* Accept style tuples in `title` argument annotation of `Choice`.\n* Added `default` for select and `initial_choice` for checkbox prompts.\n* Removed check for choices if completer is present.\n\n</details>\n\n<details><summary>1.5.2 (16.04.2020)</summary>\n\nBug fix release.\n\n* Added `.ask_async` support for forms.\n</details>\n\n<details><summary>1.5.1 (22.01.2020)</summary>\n\nBug fix release.\n\n* Fixed `.ask_async` for questions on `prompt_toolkit==2.*`. Added tests for it.\n</details>\n\n<details><summary>1.5.0 (22.01.2020)</summary>\n\nFeature release.\n\n* Added support for prompt_toolkit 3\n* All tests will be run against prompt_toolkit 2 and 3\n* Removed support for python 3.5 (prompt_toolkit 3 does not support that anymore)\n</details>\n\n<details><summary>1.4.0 (10.11.2019)</summary>\n\nFeature release.\n\n* Added additional question type `autocomplete`\n* Allow pointer and highlight in select question type\n</details>\n\n<details><summary>1.3.0 (25.08.2019)</summary>\n\nFeature release.\n\n* Add additional options to style checkboxes and select prompts https://github.com/tmbo/questionary/pull/14\n\n</details>\n\n<details><summary>1.2.1 (19.08.2019)</summary>\n\nBug fix release.\n\n* Fixed compatibility with python 3.5.2 by removing `Type` annotation (this time for real)\n</details>\n\n<details><summary>1.2.0 (30.07.2019)</summary>\n\nFeature release.\n\n* Allow a user to pass in a validator as an instance https://github.com/tmbo/questionary/pull/10\n\n</details>\n\n<details><summary>1.1.1 (21.04.2019)</summary>\n\nBug fix release.\n\n* Fixed compatibility with python 3.5.2 by removing `Type` annotation\n\n</details>\n\n<details><summary>1.1.0 (10.03.2019)</summary>\n\nFeature release.\n\n* Added `skip_if` to questions to allow skipping questions using a flag\n\n\n</details>\n\n<details><summary>1.0.2 (23.01.2019)</summary>\n\nBug fix release.\n\n* Fixed odd behaviour if select is created without providing any choices\n  instead, we will raise a `ValueError` now. ([#6](https://github.com/tmbo/questionary/pull/6))\n\n\n</details>\n\n<details><summary>1.0.1 (12.01.2019)</summary>\n\nBug fix release, adding some convenience shortcuts.\n\n* Added shortcut keys `j` (move down^ the list) and `k` (move up) to\n  the prompts `select` and `checkbox` (fixes [#2](https://github.com/tmbo/questionary/issues/2))\n* Fixed unclosed file handle in `setup.py`\n* Fixed unnecessary empty lines moving selections to far down (fixes [#3](https://github.com/tmbo/questionary/issues/3))\n\n</details>\n\n<details><summary>1.0.0 (14.12.2018)</summary>\n\nInitial public release of the library\n\n* Added python interface\n* Added dict style question creation\n* Improved the documentation\n* More tests and automatic travis test execution\n</details>\n\n## Developer Info\n\n<details>\n<summary>Create one of the commandline recordings in the readme</summary>\n\n0. Install `brew install asciinema` and `npm install --global asciicast2gif`\n1. Run `asciinema rec`\n2. Do the thing\n3. Convert to giv `asciicast2gif -h 7 -w 120 -s 2 <recoding> output.gif`\n\n</details>\n\n<details>\n<summary>Cutting a new release</summary>\n\n0. Update the version number in `questionary/version.py` AND `pyproject.toml`\n1. Add a new section for the release in the changelog in this readme\n2. commit this changes\n3. git tag the commit with the release version \n\nTravis will build and push the updated library to pypi.\n\n</details>\n\n## License\nLicensed under the MIT License. Copyright 2020 Tom Bocklisch. [Copy of the license](LICENSE).\n\n\n[![FOSSA Status](https://app.fossa.io/api/projects/git%2Bgithub.com%2Ftmbo%2Fquestionary.svg?type=large)](https://app.fossa.io/projects/git%2Bgithub.com%2Ftmbo%2Fquestionary?ref=badge_large)\n',
    'author': 'Tom Bocklisch',
    'author_email': 'tombocklisch@gmail.com',
    'maintainer': 'Tom Bocklisch',
    'maintainer_email': 'tombocklisch@gmail.com',
    'url': 'https://github.com/tmbo/questionary',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<3.10',
}


setup(**setup_kwargs)



from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union

from prompt_toolkit.application import Application
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.styles import Style, merge_styles
from prompt_toolkit.formatted_text import FormattedText

from questionary import utils
from questionary.constants import DEFAULT_QUESTION_PREFIX, DEFAULT_STYLE
from questionary.prompts import common
from questionary.prompts.common import Choice, InquirerControl, Separator
from questionary.question import Question


def checkbox(
    message: str,
    choices: Sequence[Union[str, Choice, Dict[str, Any]]],
    default: Optional[str] = None,
    validate: Callable[[List[str]], bool] = lambda a: True,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    use_pointer: bool = True,
    initial_choice: Optional[Union[str, Choice, Dict[str, Any]]] = None,
    **kwargs: Any,
) -> Question:
    """Ask the user to select from a list of items.

    This is a multiselect, the user can choose one, none or many of the
    items.

    Args:
        message: Question text

        choices: Items shown in the selection, this can contain `Choice` or
                 or `Separator` objects or simple items as strings. Passing
                 `Choice` objects, allows you to configure the item more
                 (e.g. preselecting it or disabling it).

        default: Default return value (single value). If you want to preselect
                 multiple items, use `Choice("foo", checked=True)` instead.

        validate: Require the entered value to pass a validation. The
                  value can not be submitted until the validator accepts
                  it (e.g. to check minimum password length).

                  This should be a function accepting the input and
                  returning a boolean. An optional second return value
                  is the error message to display.

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

        use_pointer: Flag to enable the pointer in front of the currently
                     highlighted element.

        initial_choice: A value corresponding to a selectable item in the choices,
                        to initially set the pointer position to.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    merged_style = merge_styles(
        [
            DEFAULT_STYLE,
            # Disable the default inverted colours bottom-toolbar behaviour (for
            # the error message). However it can be re-enabled with a custom
            # style.
            Style([("bottom-toolbar", "noreverse")]),
            style,
        ]
    )

    if not callable(validate):
        raise ValueError("validate must be callable")

    ic = InquirerControl(
        choices, default, use_pointer=use_pointer, initial_choice=initial_choice
    )

    def get_prompt_tokens() -> List[Tuple[str, str]]:
        tokens = []

        tokens.append(("class:qmark", qmark))
        tokens.append(("class:question", " {} ".format(message)))

        if ic.is_answered:
            nbr_selected = len(ic.selected_options)
            if nbr_selected == 0:
                tokens.append(("class:answer", " done"))
            elif nbr_selected == 1:
                if isinstance(ic.get_selected_values()[0].title, list):
                    ts = ic.get_selected_values()[0].title
                    tokens.append(
                        (
                            "class:answer",
                            "".join([token[1] for token in ts]),  # type:ignore
                        )
                    )
                else:
                    tokens.append(
                        (
                            "class:answer",
                            " [{}]".format(ic.get_selected_values()[0].title),
                        )
                    )
            else:
                tokens.append(
                    ("class:answer", " done ({} selections)".format(nbr_selected))
                )
        else:
            tokens.append(
                (
                    "class:instruction",
                    " (Use arrow keys to move, "
                    "<space> to select, "
                    "<a> to toggle, "
                    "<i> to invert)",
                )
            )
        return tokens

    def get_selected_values() -> List[str]:
        return [str(c.value) if c.value else "" for c in ic.get_selected_values()]

    def perform_validation(selected_values: List[str]) -> bool:

        verdict = validate(selected_values)

        if isinstance(verdict, bool):
            valid = verdict
            error_message = FormattedText(
                [("class:validation-toolbar", "Invalid selection")]
            )
        else:
            valid, error_message = verdict

            if isinstance(error_message, str):
                error_message = FormattedText(
                    [("class:validation-toolbar", error_message)]
                )

        ic.error_message = (
            error_message if not valid and ic.submission_attempted else None
        )

        return valid

    layout = common.create_inquirer_layout(ic, get_prompt_tokens, **kwargs)

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style="class:aborting")

    @bindings.add(" ", eager=True)
    def toggle(_event):
        pointed_choice = ic.get_pointed_at().value
        if pointed_choice in ic.selected_options:
            ic.selected_options.remove(pointed_choice)
        else:
            ic.selected_options.append(pointed_choice)

        perform_validation(get_selected_values())

    @bindings.add("i", eager=True)
    def invert(_event):
        inverted_selection = [
            c.value
            for c in ic.choices
            if not isinstance(c, Separator)
            and c.value not in ic.selected_options
            and not c.disabled
        ]
        ic.selected_options = inverted_selection

        perform_validation(get_selected_values())

    @bindings.add("a", eager=True)
    def all(_event):
        all_selected = True  # all choices have been selected
        for c in ic.choices:
            if (
                not isinstance(c, Separator)
                and c.value not in ic.selected_options
                and not c.disabled
            ):
                # add missing ones
                ic.selected_options.append(c.value)
                all_selected = False
        if all_selected:
            ic.selected_options = []

        perform_validation(get_selected_values())

    @bindings.add(Keys.Down, eager=True)
    @bindings.add("j", eager=True)
    def move_cursor_down(_event):
        ic.select_next()
        while not ic.is_selection_valid():
            ic.select_next()

    @bindings.add(Keys.Up, eager=True)
    @bindings.add("k", eager=True)
    def move_cursor_up(_event):
        ic.select_previous()
        while not ic.is_selection_valid():
            ic.select_previous()

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):

        selected_values = get_selected_values()
        ic.submission_attempted = True

        if perform_validation(selected_values):
            ic.is_answered = True
            event.app.exit(result=selected_values)

    @bindings.add(Keys.Any)
    def other(_event):
        """Disallow inserting other text. """
        pass

    return Question(
        Application(
            layout=layout,
            key_bindings=bindings,
            style=merged_style,
            **utils.used_kwargs(kwargs, Application.__init__),
        )
    )



import os
from typing import (
    Any,
    Callable,
    Iterable,
    List,
    Optional,
    Tuple,
)

from prompt_toolkit.completion import CompleteEvent, Completion, PathCompleter
from prompt_toolkit.document import Document
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.key_binding.bindings.completion import (
    display_completions_like_readline,
)
from prompt_toolkit.key_binding.key_processor import KeyPressEvent
from prompt_toolkit.keys import Keys
from prompt_toolkit.lexers import SimpleLexer
from prompt_toolkit.shortcuts.prompt import CompleteStyle, PromptSession
from prompt_toolkit.styles import Style, merge_styles

from questionary.constants import DEFAULT_QUESTION_PREFIX, DEFAULT_STYLE
from questionary.prompts.common import build_validator
from questionary.question import Question


class GreatUXPathCompleter(PathCompleter):
    def get_completions(
        self, document: Document, complete_event: CompleteEvent
    ) -> Iterable[Completion]:
        """Get completions.

        Wraps `PathCompleter`. Makes sure completions for directories end with
        a path separator. Also make sure the right path separator is used."""
        completions = super(GreatUXPathCompleter, self).get_completions(
            document, complete_event
        )

        for completion in completions:
            # check if the display value ends with a path separator.
            # first check if display is properly set
            styled_display = completion.display[0]
            # styled display is a formatted text (a tuple of the text and its style)
            # second tuple entry is the text
            if styled_display[1][-1] == "/":
                # replace separator with the OS specific one
                display_text = styled_display[1][:-1] + os.path.sep
                # update the styledd display with the modified text
                completion.display[0] = (styled_display[0], display_text)
                # append the separator to the text as well - unclear why the normal
                # path completer omits it from the text. this improves UX for the
                # user, as they don't need to type the separator after auto-completing
                # a directory
                completion.text += os.path.sep
            yield completion


def path(
    message: str,
    default: str = "",
    qmark: str = DEFAULT_QUESTION_PREFIX,
    validate: Any = None,
    style: Optional[Style] = None,
    only_directories: bool = False,
    file_filter: Optional[Callable[[str], bool]] = None,
    complete_style: CompleteStyle = CompleteStyle.MULTI_COLUMN,
    **kwargs: Any,
) -> Question:
    """Prompt the user to enter a path with autocomplete help.

    Args:
        message: Question text

        default: Default return value (single value).

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        complete_style: How autocomplete menu would be shown, it could be
                        COLUMN, MULTI_COLUMN or READLINE_LIKE

        validate: Require the entered value to pass a validation. The
                  value can not be submitted until the validator accepts
                  it (e.g. to check minimum password length).

                  This can either be a function accepting the input and
                  returning a boolean, or an class reference to a
                  subclass of the prompt toolkit Validator class.

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

        only_directories: Only show directories in auto completion

        file_filter: Optional callable to filter suggested paths. Only paths
                     where the passed callable evaluates to `True` will show up in
                     the suggested paths. This does not validate the typed path, e.g.
                     it is still possible for the user to enter a path manually, even
                     though this filter evaluates to `False`. If in addition to
                     filtering suggestions you also want to validate the result, use
                     `validate` in combination with the `file_filter`.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    merged_style = merge_styles([DEFAULT_STYLE, style])

    def get_prompt_tokens() -> List[Tuple[str, str]]:
        return [("class:qmark", qmark), ("class:question", " {} ".format(message))]

    validator = build_validator(validate)

    bindings = KeyBindings()

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event: KeyPressEvent):
        if event.current_buffer.complete_state is not None:
            event.current_buffer.complete_state = None
        elif event.app.current_buffer.validate(set_cursor=True):
            # When the validation succeeded, accept the input.
            result_path = event.app.current_buffer.document.text
            if result_path.endswith(os.path.sep):
                result_path = result_path[:-1]

            event.app.exit(result=result_path)
            event.app.current_buffer.append_to_history()

    @bindings.add(os.path.sep, eager=True)
    def next_segment(event: KeyPressEvent):
        b = event.app.current_buffer

        if b.complete_state:
            b.complete_state = None

        current_path = b.document.text
        if not current_path.endswith(os.path.sep):
            b.insert_text(os.path.sep)

        b.start_completion(select_first=False)

    p = PromptSession(
        get_prompt_tokens,
        lexer=SimpleLexer("class:answer"),
        style=merged_style,
        completer=GreatUXPathCompleter(
            only_directories=only_directories, file_filter=file_filter, expanduser=True
        ),
        validator=validator,
        complete_style=complete_style,
        key_bindings=bindings,
        **kwargs,
    )
    p.default_buffer.reset(Document(default))

    return Question(p.app)



from typing import (
    Any,
    Callable,
    Dict,
    List,
    Optional,
    Tuple,
    Union,
    Iterable,
)

from prompt_toolkit.completion import CompleteEvent, Completer, Completion
from prompt_toolkit.document import Document
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit.shortcuts.prompt import PromptSession, CompleteStyle
from prompt_toolkit.styles import Style, merge_styles
from prompt_toolkit.lexers import SimpleLexer

from questionary.constants import DEFAULT_QUESTION_PREFIX, DEFAULT_STYLE
from questionary.prompts.common import build_validator
from questionary.question import Question


class WordCompleter(Completer):
    choices_source: Union[List[str], Callable[[], List[str]]]
    ignore_case: bool
    meta_information: Dict[str, Any]
    match_middle: bool

    def __init__(
        self,
        choices: Union[List[str], Callable[[], List[str]]],
        ignore_case: bool = True,
        meta_information: Optional[Dict[str, Any]] = None,
        match_middle: bool = True,
    ) -> None:
        self.choices_source = choices
        self.ignore_case = ignore_case
        self.meta_information = meta_information or {}
        self.match_middle = match_middle

    def _choices(self) -> Iterable[str]:
        return (
            self.choices_source()
            if callable(self.choices_source)
            else self.choices_source
        )

    def _choice_matches(self, word_before_cursor: str, choice: str) -> int:
        """Match index if found, -1 if not. """

        if self.ignore_case:
            choice = choice.lower()

        if self.match_middle:
            return choice.find(word_before_cursor)
        elif choice.startswith(word_before_cursor):
            return 0
        else:
            return -1

    @staticmethod
    def _display_for_choice(choice: str, index: int, word_before_cursor: str) -> HTML:
        return HTML("{}<b><u>{}</u></b>{}").format(
            choice[:index],
            choice[index : index + len(word_before_cursor)],
            choice[index + len(word_before_cursor) : len(choice)],
        )

    def get_completions(
        self, document: Document, complete_event: CompleteEvent
    ) -> Iterable[Completion]:
        choices = self._choices()

        # Get word/text before cursor.
        word_before_cursor = document.text_before_cursor

        if self.ignore_case:
            word_before_cursor = word_before_cursor.lower()

        for choice in choices:
            index = self._choice_matches(word_before_cursor, choice)
            if index == -1:
                # didn't find a match
                continue

            display_meta = self.meta_information.get(choice, "")
            display = self._display_for_choice(choice, index, word_before_cursor)

            yield Completion(
                choice,
                start_position=-len(choice),
                display=display.formatted_text,
                display_meta=display_meta,
                style="class:answer",
                selected_style="class:selected",
            )


def autocomplete(
    message: str,
    choices: List[str],
    default: str = "",
    qmark: str = DEFAULT_QUESTION_PREFIX,
    completer: Optional[Completer] = None,
    meta_information: Optional[Dict[str, Any]] = None,
    ignore_case: bool = True,
    match_middle: bool = True,
    complete_style: CompleteStyle = CompleteStyle.COLUMN,
    validate: Any = None,
    style: Optional[Style] = None,
    **kwargs: Any,
) -> Question:
    """Prompt the user to enter a message with autocomplete help.

    Args:
        message: Question text

        choices: Items shown in the selection, this contains items as strings

        default: Default return value (single value).

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        completer: A prompt_toolkit `Completer` implementation. If not set, a
                questionary completer implementation will be used.

        meta_information: A dictionary with information/anything about choices.

        ignore_case: If true autocomplete would ignore case.

        match_middle: If true autocomplete would search in every string position
                      not only in string begin.

        complete_style: How autocomplete menu would be shown, it could be
                        COLUMN, MULTI_COLUMN or READLINE_LIKE

        validate: Require the entered value to pass a validation. The
                  value can not be submitted until the validator accepts
                  it (e.g. to check minimum password length).

                  This can either be a function accepting the input and
                  returning a boolean, or an class reference to a
                  subclass of the prompt toolkit Validator class.

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """

    merged_style = merge_styles([DEFAULT_STYLE, style])

    def get_prompt_tokens() -> List[Tuple[str, str]]:
        return [("class:qmark", qmark), ("class:question", " {} ".format(message))]

    def get_meta_style(meta: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:
        if meta:
            for key in meta:
                meta[key] = HTML("<text>{}</text>").format(meta[key])

        return meta

    validator = build_validator(validate)

    if completer is None:
        if not choices:
            raise ValueError("No choices is given, you should use Text question.")
        # use the default completer
        completer = WordCompleter(
            choices,
            ignore_case=ignore_case,
            meta_information=get_meta_style(meta_information),
            match_middle=match_middle,
        )

    p = PromptSession(
        get_prompt_tokens,
        lexer=SimpleLexer("class:answer"),
        style=merged_style,
        completer=completer,
        validator=validator,
        complete_style=complete_style,
        **kwargs,
    )
    p.default_buffer.reset(Document(default))

    return Question(p.app)



from typing import Sequence, Optional, Any, Union, Dict

from prompt_toolkit.styles import Style

from questionary.constants import DEFAULT_QUESTION_PREFIX
from questionary.prompts import select
from questionary.prompts.common import Choice
from questionary.question import Question


def rawselect(
    message: str,
    choices: Sequence[Union[str, Choice, Dict[str, Any]]],
    default: Optional[str] = None,
    qmark: str = DEFAULT_QUESTION_PREFIX,
    style: Optional[Style] = None,
    **kwargs: Any,
) -> Question:
    """Ask the user to select one item from a list of choices using shortcuts.

    The user can only select one option.

    Args:
        message: Question text

        choices: Items shown in the selection, this can contain `Choice` or
                 or `Separator` objects or simple items as strings. Passing
                 `Choice` objects, allows you to configure the item more
                 (e.g. preselecting it or disabeling it).

        default: Default return value (single value).

        qmark: Question prefix displayed in front of the question.
               By default this is a `?`

        style: A custom color and style for the question parts. You can
               configure colors as well as font types for different elements.

    Returns:
        Question: Question instance, ready to be prompted (using `.ask()`).
    """
    return select.select(
        message,
        choices,
        default,
        qmark,
        style,
        use_shortcuts=True,
        use_arrow_keys=False,
        **kwargs,
    )
